//------------------------------------------------------------------------------
//	$Id: pairs.js,v 1.9 2018/12/15 09:45:09 wolf Exp $
//------------------------------------------------------------------------------
//	Einfaches Gedaechtnis-Spiel
//------------------------------------------------------------------------------
//	This program is free software. It comes without any warranty, to
//	the extent permitted by applicable law. You can redistribute it
//	and/or modify it under the terms of the Do What The Fuck You Want
//	To Public License, Version 2, as published by Sam Hocevar. See
//	http://www.wtfpl.net/ for more details.
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

function addListener (element, eventName, handler) {

	if (element.addEventListener) {
		element.addEventListener(eventName, handler, false);
	} else if (window.document.attachEvent) {
		element.attachEvent('on'+eventName, handler);
	} else {
		window.alert ('Neither addEventListener nor attachEvent defined.');
	}
}

addListener(window, 'load', function() {

	function createPairs (container) {

		var nFields   = parseInt(container.getAttribute('data-pairs-n-fields'), 10);
		var nTiles    = parseInt(container.getAttribute('data-pairs-n-tiles'), 10);
		var tileSize  = parseInt(container.getAttribute('data-pairs-tile-size'), 10);
		var maxPerRow = parseInt(container.getAttribute('data-pairs-max-per-row'), 10) || 6;
		var tilePath  = container.getAttribute('data-pairs-tile-path');
		var message   = container.getAttribute('data-pairs-message-success') ||
				'Geschafft!\nDu hast {f} Karten mit {c} Klicks aufgedeckt.';
		var label     = container.getAttribute('data-pairs-label-reset') ||
				'Neues Spiel';

		if (!nFields || !nTiles || !tilePath) return false;

		// config by location hash

		if (window.location && window.location.hash) {

			var match;
			match = window.location.hash.match(/[#&]f=([1-9]\d*)([&].*)?$/);
			if (match) nFields = parseInt(match[1], 10);

			match = window.location.hash.match(/[#&]s=([1-9]\d+)([&].*)?$/);
			if (match) tileSize = parseInt(match[1], 10);

			match = window.location.hash.match(/[#&]m=([1-9]\d?)([&].*)?$/);
			if (match) maxPerRow = parseInt(match[1], 10);
		}

		// sanity

		nFields = Math.max (2, Math.min (1000, nFields&~1));

		// container

		if (tileSize && maxPerRow) {

			container.style.maxWidth = tileSize * maxPerRow + 'px';
		}

		// tiles

		var emptyTile  = tilePath.replace(/%/, '00');

		var tiles = [];
		for (var t=1; t<=nTiles; t++) {
			tiles.push(tilePath.replace(/%/, t>9? t : '0'+t));
		}

		// board

		var board;		// virtual
		var fields = [];	// rendered
		var found  = [];
		var nFound;

		// logic

		var pivot1;
		var pivot2;
		var nClicks;

		function show(msg) {}

		function click (idx) {

			// fields[idx].setAttribute('src', board[idx]);

			if (idx===pivot1 || idx===pivot2 || found[idx]) return false;
			++nClicks;

			// document.forms["wichtel"].tries.value=++Try;

			if (pivot2!==null) fields[pivot2].setAttribute('src', emptyTile);

			pivot2=pivot1;
			pivot1=idx;

			fields[pivot1].setAttribute('src', board[pivot1]);

			if (board[pivot1] != board[pivot2]) {

				// document.forms["wichtel"].msg.value=messageSearch;
				return false;
			}

			found[pivot1] = true;
			found[pivot2] = true;
			pivot1 = null;
			pivot2 = null;

			nFound += 2;
			if (nFound < nFields) return false;

			window.setTimeout(function() {
				window.alert (message.replace(/{f}/g, nFields).replace(/{c}/g, nClicks));
			}, 300);
			return false;
		}

		// reset

		function resetBoard() {

			pivot1 = null;
			pivot2 = null;
			found  = [];
			nFound = 0;
			nClicks = 0;

			var positions = [];
			for (var i=0; i<nFields; i++) {
				positions[i] = i;
				fields[i].setAttribute('src', emptyTile);
			}
			var myTiles = [];
			board = [];
			while (positions.length>=2) {
				var pi = Math.random() * positions.length;
				var p1 = positions.splice(pi, 1)[0];
				pi = Math.random() * positions.length;
				var p2 = positions.splice(pi, 1)[0];
				if (!myTiles.length) myTiles = tiles.slice();
				var ti = Math.random() * myTiles.length;
				var tile = myTiles.splice(ti, 1)[0];
				board[p1] = board[p2] = tile;
			}
		}

		// event handler

		var oldScrollX;
		var oldScrollY;

		function onclick(evt) {

			click(parseInt(this.getAttribute('data-f'), 10));
		}

		function ontouchend(evt) {

			if (oldScrollX != document.documentElement.scrollLeft) return;
			if (oldScrollY != document.documentElement.scrollTop) return;
		}

		function ontouchstart(evt) {

			oldScrollX = document.documentElement.scrollLeft;
			oldScrollY = document.documentElement.scrollTop;
		}

		// create board

		var tileBox = document.createElement('div');
		for (var f=0; f<nFields; f++) {

			var img = document.createElement('img');
			img.setAttribute('src', emptyTile);
			img.setAttribute('data-f', f);
			img.style.verticalAlign='middle';

			addListener(img, 'touchstart', ontouchstart);
			addListener(img, 'touchend', ontouchend);
			addListener(img, 'click', onclick);

			tileBox.appendChild(img);
			fields.push(img);
		}

		// create controls

		var controlBox = document.createElement('p');

		var resetButton = document.createElement('button');
		resetButton.appendChild(document.createTextNode(label));
		addListener(resetButton, 'click', resetBoard);
		controlBox.appendChild(resetButton);

		// render

		while (container.firstChild) container.removeChild(container.firstChild);

		container.appendChild(tileBox);
		container.appendChild(controlBox);

		resetBoard();

		return true;
	}

	var containers = document.getElementsByClassName('pairs');
	for (var c=0; c<containers.length; c++) {

		createPairs(containers[c]);
	}
});

//------------------------------------------------------------------------------
//	$Id: pairs.js,v 1.9 2018/12/15 09:45:09 wolf Exp $
//------------------------------------------------------------------------------
